/*
 * audio.cpp: Audio frame handling
 *
 * See the main source file 'main.cpp' for copyright information and
 * how to reach the author.
 */

#include "audio.h"
#include "util.h"
#include "tools.h"

#include <stdlib.h>
#include <stdio.h>

FILE *rawFile;
FILE *mpegFile;
FILE *ac3File;

cRawAudioFrame::cRawAudioFrame(unsigned char * data, unsigned int length, int channels)
:	data(data),
	length(length),
	channels(channels)
{
//	fwrite(data, length, 1, rawFile);
}

cRawAudioFrame::~cRawAudioFrame()
{
	if (this->length > 0)
		delete[] this->data;
}

cMPEGAudioFrame::cMPEGAudioFrame(unsigned char * data, unsigned int length)
:	cBitStream(data, length)
{
//	fwrite(data, length, 1, mpegFile);
	Parse();
}

cMPEGAudioFrame::~cMPEGAudioFrame()
{
}

cRawAudioFrame * cMPEGAudioFrame::Decode(AVCodecContext * c)
{
	int out_size, len;
	unsigned char *outbuf;
	cRawAudioFrame *frame = NULL;

	outbuf = new unsigned char[AVCODEC_MAX_AUDIO_FRAME_SIZE];

	len = avcodec_decode_audio(c, (short *) outbuf, &out_size,
								this->data, this->length + 1);
	if (len < 0)
	{
		esyslog("Error while decoding\n");
		return NULL;
	}
	if (out_size > 0)
	{
		unsigned char *buf = new unsigned char[out_size];
		memcpy(buf, outbuf, out_size);
		frame = new cRawAudioFrame(buf, out_size, c->channels);
	}

	delete[] outbuf;

	return frame;
}

void cMPEGAudioFrame::Parse()
{
	int lsf;
	int mpeg25;
	
	this->SeekBits(11);
	this->version = GetBits(2);
	lsf = (this->version == 2 || this->version == 0) ? 1 : 0;
	mpeg25 = (this->version < 2) ? 1 : 0;
	this->layer = 4 - GetBits(2);
	this->SeekBits(1);
	this->bitRate = kBitrateTab[lsf][this->layer - 1][GetBits(4)];
	this->samplingRate = kFreqTab[GetBits(2)] >> (lsf + mpeg25);
	this->padding = GetBits(1);
	this->SeekBits(1);
	this->channelMode = GetBits(2);
	this->modeExtension = GetBits(2);
	this->copyright = GetBits(1);
	this->original = GetBits(1);
	this->emphasis = GetBits(2);
	switch(this->layer)
	{
	case 1:
		this->frameSize =
			((this->bitRate * 12000) / this->samplingRate + this->padding) << 2;
		break;
	case 2:
		this->frameSize =
			(this->bitRate * 144000) / this->samplingRate + this->padding;
		break;
	case 3:
		this->frameSize =
			(this->bitRate * 144000) / (this->samplingRate << lsf) + this->padding;
		break;
	}
}

cAC3AudioFrame::cAC3AudioFrame(unsigned char * data, unsigned int length)
:	cBitStream(data, length)
{
//	fwrite(data, length, 1, ac3File);
	Parse();
}

cAC3AudioFrame::~cAC3AudioFrame()
{
}

void cAC3AudioFrame::Parse()
{
	static unsigned char halfrate[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3};
	static int rate[] = { 32,  40,  48,  56,  64,  80,  96, 112,
						 128, 160, 192, 224, 256, 320, 384, 448,
						 512, 576, 640};
	int samplingRateCode;
	int frameSizeCode;
	int version;
	int audioService;
	int audioCodingMode;
	int half;
	int bitrate;

	if (this->length < 7)
		return;
	// Syncword und CRC ueberspringen
	GetBits(32);
	samplingRateCode = GetBits(2);
	frameSizeCode = GetBits(6);
	version = GetBits(5);
	audioService = GetBits(3);
	audioCodingMode = GetBits(3);

	half = halfrate[version];
	bitrate = rate[frameSizeCode >> 1];
	this->bitRate = (bitrate * 1000) >> half;
	switch (samplingRateCode)
	{
	case 0:
		this->samplingRate = 48000 >> half;
		this->frameSize = 4 * bitrate;
		break;
	case 1:
		this->samplingRate = 44100 >> half;
		this->frameSize = 2 * (320 * bitrate / 147 + (frameSizeCode & 1));
	case 2:
		this->samplingRate = 32000 >> half;
		this->frameSize = 6 * bitrate;
	default:
		this->samplingRate = 0;
		this->frameSize = 0;
	}
	if (audioCodingMode == 2)
		if (GetBits(2) == 2)
			audioCodingMode = kA52_Dolby;
	this->channelMode = audioCodingMode;
}

