/*
 * scanner.cpp: Main logic and analysis
 *
 * See the main source file 'main.cpp' for copyright information and
 * how to reach the author.
 */

#include "global.h"
#include "scanner.h"
#include "util.h"
#include <sys/time.h>

static const int kStateProgramme		= 1;
static const int kStatePreview			= 2;
static const int kStateCommercial		= 3;

cCommercialScanner::cCommercialScanner(const char * fileName)
{
	char file[1000];
	
	strcpy(this->fileName, fileName);
	this->recFile = new cFileName(fileName, false);
	this->recIndex = new cIndexFile(fileName, false);
	sprintf(file, "%s/test1.log", kFilePath);
	this->logFile = fopen(file, "ab");
	fprintf(this->logFile, "%s\n", fileName);

	/* find the mpeg audio decoder */
	this->audioCodec = avcodec_find_decoder(CODEC_ID_MP3LAME);
	if (!audioCodec)
	{
		esyslog("audio codec not found\n");
	}
	for (int i = 0; i < 16; i++)
	{
		this->audioCodecContext[i] = avcodec_alloc_context();
		/* open it */
		if (avcodec_open(this->audioCodecContext[i], this->audioCodec) < 0)
		{
			esyslog("could not open codec\n");
		}
	}
}

cCommercialScanner::~cCommercialScanner()
{
	int i;
	
	for (i = 0; i < 16; i++)
	{
		avcodec_close(this->audioCodecContext[i]);
		free(this->audioCodecContext[i]);
	}
	delete this->recFile;
	delete this->recIndex;
	fclose(logFile);
}

void cCommercialScanner::Process()
{
	unsigned char fileNumber;
	int fileOffset;
	unsigned char pictureType;
	int length;
	int file;
	int first;
	int last;
	int step = 0;
	int nextProgress = 0;
	char str[100];
	int index;
	int gopIndex;

	cPESFrame *pesFrame;
	cMPEGVideoFrame *mpegVideo;
	cMPEGAudioFrame *mpegAudio;
	cRawVideoFrame *rawVideo;
	cRawAudioFrame *rawAudio;
	cAC3AudioFrame *ac3Audio;
	
	first = 0;
	last = this->recIndex->Last();

	if (kShowVDRMessages)
	{
		step = (last - first) / 20;
		nextProgress = step;
		VDRMessage("Starting Commercial Scan");
	}
	gopIndex = 0;
	for (index = first; index < last; index++)
	{
		if (kShowVDRMessages)
		{
			if (index == nextProgress)
			{
				sprintf(str, "Commercial Scan %d%%", ((index - first) / step) * 5);
				VDRMessage(str);
				nextProgress += step;
			}
		}
		if (kVerbose)
		{
			printf(" %06d\r", index);
			fflush(stdout);
		}
		if (this->recIndex->Get(index, &fileNumber, &fileOffset,
								&pictureType, &length))
		{
			file = this->recFile->SetOffset(fileNumber, fileOffset);
			if (file != -1)
			{
				length = ReadFrame(file, frameBuffer, length, 256 * 1024);
				pesFrame = new cPESFrame(frameBuffer, length, pictureType);
				CheckPESFrameFeatures(index, gopIndex, pesFrame);
				mpegVideo = pesFrame->GetVideoFrame(0);
				CheckMPEGVideoFeatures(index, gopIndex, mpegVideo);
				if (mpegVideo->FrameType() == (int) kFrameTypeI)
				{
					rawVideo = mpegVideo->Decode();
					if (rawVideo)
					{
						CheckRawVideoFeatures(index, gopIndex, rawVideo);
						delete rawVideo;
					}
					gopIndex++;
				}
				delete mpegVideo;

				for (int i = 0; i < pesFrame->GetAudioFrameCount(0); i++)
				{
					mpegAudio = pesFrame->GetAudioFrame(0, i);
					CheckMPEGAudioFeatures(index, gopIndex, mpegAudio);
					rawAudio = mpegAudio->Decode(this->audioCodecContext[0]);
					if (rawAudio)
					{
						CheckRawAudioFeatures(index, gopIndex, rawAudio);
						delete rawAudio;
					}
					delete mpegAudio;
				}

				for (int i = 0; i < pesFrame->GetAC3AudioFrameCount(0); i++)
				{
					ac3Audio = pesFrame->GetAC3AudioFrame(0, i);
					CheckAC3AudioFeatures(index, gopIndex, ac3Audio);
					delete ac3Audio;
				}

				delete pesFrame;
			}
		}
	}
	if (kShowVDRMessages)
	{
		VDRMessage("Commercial Scan finished");
	}

	tLogoChange changeL;
	changeL.index = index;
	changeL.logoIndex = -2;
	changeL.logoPosition.x = -2;
	changeL.logoPosition.y = -2;
	logoChanges.push_back(changeL);

	tBorderChange changeB;
	changeB.index = index;
	changeB.border.top = -2;
	changeB.border.bottom = -2;
	changeB.border.left = -2;
	changeB.border.right = -2;
	borderChanges.push_back(changeB);
	
	tBlackWhiteChange changeBW;
	changeBW.index = index;
	changeBW.bwState = -2;
	bwChanges.push_back(changeBW);
	
	tAC3ChannelChange changeAC3;
	changeAC3.index = index;
	changeAC3.channelMode = -2;
	ac3Changes.push_back(changeAC3);

	tMonoStereoChange changeMS;
	changeMS.index = index;
	changeMS.channels = -2;
	monostereoChanges.push_back(changeMS);

	CheckBorderInsymmetries();

	Analysis();

	SetCuttingMarks();

	DumpChanges();
}

void cCommercialScanner::CheckPESFrameFeatures(int index, int gopIndex, void * pesFrame)
{
}

void cCommercialScanner::CheckMPEGVideoFeatures(int index, int gopIndex, void * mpegVideo)
{
}

void cCommercialScanner::CheckRawVideoFeatures(int index, int gopIndex, void * rawVideo)
{
	logo.Refresh(rawVideo);
	if (logo.Changed())
	{
		tLogoChange change;
		tLogoChange prevChange;
		tLogoChange prev2Change;

		change.index = index;
		change.gopIndex = gopIndex;
		change.logoIndex = logo.LogoIndex();
		change.logoPosition = logo.Position();
		if (logoChanges.size() < 2)
		{
			logoChanges.push_back(change);
		}
		else
		{
			prevChange = logoChanges.back();
			if (CompareLogos(prevChange, change) == 0)
			{
				logoChanges.pop_back();
				prevChange.logoIndex = change.logoIndex;
				prevChange.logoPosition = change.logoPosition;
				logoChanges.push_back(prevChange);
			}
			else
			{
				if (change.gopIndex - prevChange.gopIndex < 5)
				{
					logoChanges.pop_back();
					prev2Change = logoChanges.back();
					if (CompareLogos(prev2Change, change) == 0)
					{
						logoChanges.pop_back();
						prev2Change.logoIndex = change.logoIndex;
						prev2Change.logoPosition = change.logoPosition;
						logoChanges.push_back(prev2Change);
					}
					else
					{
						logoChanges.push_back(prevChange);
						logoChanges.push_back(change);
					}
				}
				else
				{
					logoChanges.push_back(change);
				}
			}
		}
	}

	border.Refresh(rawVideo);
	if (border.Changed())
	{
		tBorderChange change;
		tBorderChange prevChange;
		tBorderChange prev2Change;

		change.index = index;
		change.gopIndex = gopIndex;
		change.border = border.Border();
		if (borderChanges.size() < 2)
		{
			borderChanges.push_back(change);
		}
		else
		{
			prevChange = borderChanges.back();
			if (CompareBorders(prevChange.border, change.border) == 0)
			{
				borderChanges.pop_back();
				prevChange.border = change.border;
				borderChanges.push_back(prevChange);
			}
			else
			{
				if (change.gopIndex - prevChange.gopIndex < 8)
				{
					borderChanges.pop_back();
					prev2Change = borderChanges.back();
					if (CompareBorders(prev2Change.border, change.border) == 0)
					{
						borderChanges.pop_back();
						prev2Change.border = change.border;
						borderChanges.push_back(prev2Change);
					}
					else
					{
						borderChanges.push_back(prevChange);
						borderChanges.push_back(change);
					}
				}
				else
				{
					borderChanges.push_back(change);
				}
			}
		}
	}

	blackwhite.Refresh(rawVideo);
	if (blackwhite.Changed())
	{
		tBlackWhiteChange change;
		tBlackWhiteChange prevChange;
		tBlackWhiteChange prev2Change;

		change.index = index;
		change.gopIndex = gopIndex;
		change.bwState = blackwhite.State();
		if (bwChanges.size() < 2)
		{
			bwChanges.push_back(change);
		}
		else
		{
			prevChange = bwChanges.back();
			if (prevChange.bwState == change.bwState)
			{
			}
			else
			{
				if (change.gopIndex - prevChange.gopIndex < 3)
				{
					bwChanges.pop_back();
					prev2Change = bwChanges.back();
					if (prev2Change.bwState == change.bwState)
					{
					}
					else
					{
						// kann eigentlich nicht passieren,
						// da nur 2 verschiedene States
						prevChange.bwState = change.bwState;
						bwChanges.push_back(prevChange);
					}
				}
				else
				{
					bwChanges.push_back(change);
				}
			}
		}
	}
}

void cCommercialScanner::CheckMPEGAudioFeatures(int index, int gopIndex, void * mpegAudio)
{
}

void cCommercialScanner::CheckRawAudioFeatures(int index, int gopIndex, void * rawAudio)
{
	monostereo.Refresh(rawAudio);
	if (monostereo.Changed())
	{
		tMonoStereoChange change;
		tMonoStereoChange prevChange;
		tMonoStereoChange prev2Change;

		change.index = index;
		change.gopIndex = gopIndex;
		change.channels = monostereo.Channels();
		if (monostereoChanges.size() < 2)
		{
			monostereoChanges.push_back(change);
		}
		else
		{
			prevChange = monostereoChanges.back();
			if (prevChange.channels == change.channels)
			{
			}
			else
			{
				if (change.index - prevChange.index < 10)
				{
					monostereoChanges.pop_back();
					prev2Change = monostereoChanges.back();
					if (prev2Change.channels == change.channels)
					{
					}
				}
				else
				{
					monostereoChanges.push_back(change);
				}
			}
		}
	}
}

void cCommercialScanner::CheckAC3AudioFeatures(int index, int gopIndex, void * ac3Audio)
{
	ac3channel.Refresh(ac3Audio);
	if (ac3channel.Changed())
	{
		tAC3ChannelChange change;
		change.index = index;
		change.gopIndex = gopIndex;
		change.channelMode = ac3channel.ChannelMode();
		ac3Changes.push_back(change);
    }
}

void cCommercialScanner::CheckBorderInsymmetries()
{
	list<tBorderChange>::iterator itBorder;
	list<tBorderChange>::iterator itPrevBorder;
	list<tBorderChange>::iterator itNextBorder;
	int prev;
	int next;

	itBorder = borderChanges.begin();
	while (itBorder != borderChanges.end())
	{
		if (itBorder == borderChanges.begin())
		{
			itBorder++;
			continue;
		}
		if (abs(itBorder->border.top - itBorder->border.bottom) > 15)
		{
			itPrevBorder = itBorder;
			itPrevBorder--;
			itNextBorder = itBorder;
			itNextBorder++;
			if (itNextBorder == borderChanges.end())
			{
				itBorder->border.top = itPrevBorder->border.top;
				itBorder->border.bottom = itPrevBorder->border.bottom;
				if (CompareBorders(itPrevBorder->border, itBorder->border) == 0)
				{
					borderChanges.erase(itBorder);
					itBorder = itPrevBorder;
				}
			}
			else
			{
				prev = abs(itPrevBorder->border.left - itBorder->border.left) +
					   abs(itPrevBorder->border.right - itBorder->border.right);
				next = abs(itNextBorder->border.left - itBorder->border.left) +
					   abs(itNextBorder->border.right - itBorder->border.right);
				if (prev <= next)
				{
					itBorder->border.top = itPrevBorder->border.top;
					itBorder->border.bottom = itPrevBorder->border.bottom;
					if (CompareBorders(itPrevBorder->border, itBorder->border) == 0)
					{
						borderChanges.erase(itBorder);
						itBorder = itPrevBorder;
					}
				}
				else
				{
					itBorder->border.top = itNextBorder->border.top;
					itBorder->border.bottom = itNextBorder->border.bottom;
					if (CompareBorders(itNextBorder->border, itBorder->border) == 0)
					{
						borderChanges.erase(itNextBorder);
					}
				}
			}
			itBorder++;
			continue;
		}
		if (abs(itBorder->border.left - itBorder->border.right) > 15)
		{
			itPrevBorder = itBorder;
			itPrevBorder--;
			itNextBorder = itBorder;
			itNextBorder++;
			if (itNextBorder == borderChanges.end())
			{
				itBorder->border.left = itPrevBorder->border.left;
				itBorder->border.right = itPrevBorder->border.right;
				if (CompareBorders(itPrevBorder->border, itBorder->border) == 0)
				{
					borderChanges.erase(itBorder);
					itBorder = itPrevBorder;
				}
			}
			else
			{
				prev = abs(itPrevBorder->border.top - itBorder->border.top) +
					   abs(itPrevBorder->border.bottom - itBorder->border.bottom);
				next = abs(itNextBorder->border.top - itBorder->border.top) +
					   abs(itNextBorder->border.bottom - itBorder->border.bottom);
				if (prev <= next)
				{
					itBorder->border.left = itPrevBorder->border.left;
					itBorder->border.right = itPrevBorder->border.right;
					if (CompareBorders(itPrevBorder->border, itBorder->border) == 0)
					{
						borderChanges.erase(itBorder);
						itBorder = itPrevBorder;
					}
				}
				else
				{
					itBorder->border.left = itNextBorder->border.left;
					itBorder->border.right = itNextBorder->border.right;
					if (CompareBorders(itNextBorder->border, itBorder->border) == 0)
					{
						borderChanges.erase(itBorder);
						itBorder = itPrevBorder;
					}
				}
			}
			itBorder++;
			continue;
		}
		itBorder++;
	}

	// Doppelte rausschmeissen
	itBorder = borderChanges.begin();
	while (itBorder != borderChanges.end())
	{
		itNextBorder = itBorder;
		itNextBorder++;
		if (itNextBorder != borderChanges.end())
		{
			if (CompareBorders(itBorder->border, itNextBorder->border) == 0)
			{
				borderChanges.erase(itNextBorder);
			}
		}
		itBorder++;
	}
}

void cCommercialScanner::Analysis()
{
	int state;

	list <tBorderChange>::iterator itBorder;
	list <tLogoChange>::iterator itLogo;
	list <tLogoChange>::iterator itPrevLogo;
	list <tLogoChange>::iterator itNextLogo;
	tLogoChange lastLogo;

	if (this->AC3ChangesRelevant())
	{
		list <tAC3ChannelChange>::iterator itAC3;

		itAC3 = ac3Changes.begin();
		if (itAC3->channelMode == 2)
		{
			state = kStateCommercial;
		}
		else
		{
			state = kStateProgramme;
		}
		AddChange(itAC3->index, state);
		itAC3++;
		while (itAC3 != ac3Changes.end())
		{
			if (itAC3->channelMode == -2)
				break;
			if (state == kStateCommercial)
			{
				if (itAC3->channelMode != 2)
				{
					state = kStateProgramme;
					AddChange(itAC3->index, state);
				}
			}
			else if (state == kStateProgramme)
			{
				if (itAC3->channelMode == 2)
				{
					state = kStateCommercial;
					AddChange(itAC3->index, state);
				}
			}
			itAC3++;
		}
	}
	else if (this->BWChangesRelevant())
	{
		list <tBlackWhiteChange>::iterator itBW;
		list <tBlackWhiteChange>::iterator itNextBW;

		itBW = bwChanges.begin();
		itNextBW = itBW;
		itNextBW++;
		if (itBW->bwState == 1)
		{
			state = kStateCommercial;
		}
		else
		{
			state = kStateProgramme;
		}
		AddChange(itBW->index, state);
		itBW++;
		while (itBW != bwChanges.end())
		{
			itNextBW = itBW;
			itNextBW++;
			if (itBW->bwState == -2)
				break;
			if (state == kStateCommercial)
			{
				if (itBW->bwState == 0)
				{
					if (itNextBW->index - itBW->index > 1000)
					{
						state = kStateProgramme;
						AddChange(itBW->index, state);
					}
				}
			}
			else if (state == kStateProgramme)
			{
				if (itBW->bwState == 1)
				{
					state = kStateCommercial;
					AddChange(itBW->index, state);
				}
			}
			itBW++;
		}
	}
	else
	{
		if (logoChanges.size() < 2)
		{
			return;
		}

		itLogo = logoChanges.begin();
		itNextLogo = itLogo;
		itNextLogo++;

		if (itLogo->logoIndex == -1)
		{
			state = kStateCommercial;
		}
		else
		{
			if (itNextLogo->index - itLogo->index < 1000)
			{
				state = kStatePreview;
			}
			else
			{
				state = kStateProgramme;
				lastLogo.logoIndex = itLogo->logoIndex;
				lastLogo.logoPosition = itLogo->logoPosition;
			}
		}
		AddChange(itLogo->index, state);
		itLogo++;
		while (itLogo != logoChanges.end())
		{
			itNextLogo = itLogo;
			itNextLogo++;
			if (state == kStateCommercial)
			{
				if (itNextLogo->index - itLogo->index > 1000)
				{
					state = kStateProgramme;
					itBorder = this->GetBorderAtIndex(itLogo->index);
					if (itBorder->index != itLogo->index)
						itBorder--;
					AddChange(itBorder->index, state);
					lastLogo.logoIndex = itLogo->logoIndex;
					lastLogo.logoPosition = itLogo->logoPosition;
				}
				else /*if (itNextLogo->index - itLogo->index > 100)*/
				{
					state = kStatePreview;
					AddChange(itLogo->index, state);
				}
			}
			else if (state == kStatePreview)
			{
				if (itLogo->logoIndex == -1)
				{
					if (itNextLogo->index - itLogo->index > 1000)
					{
						state = kStateCommercial;
						AddChange(itLogo->index, state);
					}
					else
					{
						itLogo++;
						itNextLogo++;
						if (itNextLogo->index - itLogo->index > 1000)
						{
							state = kStateProgramme;
							itBorder = this->GetBorderAtIndex(itLogo->index);
							if (itBorder->index != itLogo->index)
								itBorder--;
							AddChange(itBorder->index, state);
							lastLogo.logoIndex = itLogo->logoIndex;
							lastLogo.logoPosition = itLogo->logoPosition;
						}
					}
				}
				else
				{
					if (itNextLogo->index - itLogo->index > 1000)
					{
						state = kStateProgramme;
						itBorder = this->GetBorderAtIndex(itLogo->index);
						if (itBorder->index != itLogo->index)
							itBorder--;
						AddChange(itBorder->index, state);
						lastLogo.logoIndex = itLogo->logoIndex;
						lastLogo.logoPosition = itLogo->logoPosition;
					}
				}
			}
			else if (state == kStateProgramme)
			{
				if (itLogo->logoIndex == -1)
				{
					if (itNextLogo->index - itLogo->index > 1000)
					{
						state = kStateCommercial;
						AddChange(itLogo->index, state);
					}
					else
					{
						itLogo++;
						itNextLogo++;
						if (itNextLogo->index - itLogo->index > 1000)
						{
							int comp = CompareLogos(lastLogo, *itLogo);
							if (comp == 1 || comp == 2)
							{
								itLogo--;
								itNextLogo--;
								state = kStateCommercial;
								AddChange(itLogo->index, state);
							}
							else
							{
								itPrevLogo = itLogo;
								itPrevLogo--;
								int chunks = GetBorderChangeCount(itPrevLogo->index, itLogo->index);
								if (chunks > 2)
								{
									itLogo--;
									itNextLogo--;
									state = kStateCommercial;
									AddChange(itLogo->index, state);
								}
								else
								{
									state = kStateProgramme;
									lastLogo.logoIndex = itLogo->logoIndex;
									lastLogo.logoPosition = itLogo->logoPosition;
								}
							}
						}
						else
						{
							itLogo--;
							itNextLogo--;
							state = kStateCommercial;
							AddChange(itLogo->index, state);
						}
					}
				}
				else
				{
					if (itNextLogo->index - itLogo->index > 1000)
					{
						state = kStateProgramme;
						lastLogo.logoIndex = itLogo->logoIndex;
						lastLogo.logoPosition = itLogo->logoPosition;
					}
				}
			}
			itLogo++;
		}
	}
}

void cCommercialScanner::SetCuttingMarks()
{
	list <tChange>::iterator itChange;
	list <tChange>::iterator itPrevChange;
	int index;
	list <int>::iterator itMark;
	cMarks markFile;
	cMark * mark;

	itChange = changes.begin();
	if (itChange->state == kStateProgramme)
	{
		marks.push_back(itChange->index);
	}
	itPrevChange = itChange;
	itChange++;
	while (itChange != changes.end())
	{
		if (itPrevChange->state == kStateProgramme)
		{
			if (itChange->state == kStatePreview ||
				itChange->state == kStateCommercial)
			{
				index = recIndex->GetNextIFrame(itChange->index, true);
				index = recIndex->GetNextIFrame(index, true);
				marks.push_back(index);
			}
		}
		if (itPrevChange->state == kStatePreview ||
			itPrevChange->state == kStateCommercial)
		{
			if (itChange->state == kStateProgramme)
			{
				index = recIndex->GetNextIFrame(itChange->index, false);
				index = recIndex->GetNextIFrame(index, false);
				marks.push_back(index);
			}
		}
		itPrevChange++;
		itChange++;
	}
	if (changes.back().state == kStateProgramme)
		marks.push_back(logoChanges.back().index);
	if (marks.front() < 0)
		marks.front() = 0;

	markFile.Load(this->fileName);
	while ((mark = markFile.GetNext(0)))
		markFile.Del(mark);
	while ((mark = markFile.GetPrev(1)))
		markFile.Del(mark);
	for (itMark = marks.begin(); itMark != marks.end(); itMark++)
	{
		markFile.Add(*itMark);
	}
	markFile.Save();
}

bool cCommercialScanner::AC3ChangesRelevant()
{
	int sum = 0;
	list <tAC3ChannelChange>::iterator itAC3;
	list <tAC3ChannelChange>::iterator itNextAC3;

	if (ac3Changes.size() < 2)
	{
		return false;
	}
	itAC3 = ac3Changes.begin();
	while (itAC3 != ac3Changes.end())
	{
		itNextAC3 = itAC3;
		itNextAC3++;
		if (itAC3->channelMode == -2)
			break;
		if (itAC3->channelMode != 2)
		{
			sum += itNextAC3->index - itAC3->index;
		}
		itAC3++;
	}
	sum = sum * 2;
	if (sum > ac3Changes.back().index)
		return true;
	else
		return false;
}

bool cCommercialScanner::BWChangesRelevant()
{
	int sum = 0;
	list <tBlackWhiteChange>::iterator itBW;
	list <tBlackWhiteChange>::iterator itNextBW;

	if (bwChanges.size() < 2)
	{
		return false;
	}
	itBW = bwChanges.begin();
	while (itBW != bwChanges.end())
	{
		itNextBW = itBW;
		itNextBW++;
		if (itBW->state == -2)
			break;
		if (itBW->bwState == 0)
		{
			sum += itNextBW->index - itBW->index;
		}
		itBW++;
	}
	sum = sum * 2;
	if (sum > bwChanges.back().index)
		return true;
	else
		return false;
}

void cCommercialScanner::AddChange(int index, int state)
{
	tChange change;
	change.index = index;
	change.state = state;
	changes.push_back(change);
}

int cCommercialScanner::GetBorderChangeCount(int start, int end)
{
	int count;
	list <tBorderChange>::iterator itBorder;

	count = 0;
	itBorder = borderChanges.begin();
	while (itBorder->index < start)
	{
		itBorder++;
	}
	while (itBorder->index < end)
	{
		count++;
		itBorder++;
	}
	return count;
}

list <tBorderChange>::iterator cCommercialScanner::GetBorderAtIndex(int index)
{
	list <tBorderChange>::iterator itBorder;
	itBorder = borderChanges.begin();
	while (itBorder->index < index)
	{
		itBorder++;
	}
	return itBorder;
}

list <tBorderChange>::iterator cCommercialScanner::GetBorderChunk(int start, int end, int minLength)
{
	list <tBorderChange>::iterator itBorder;
	list <tBorderChange>::iterator itNextBorder;

	itBorder = borderChanges.begin();
	while (itBorder->index < start)
	{
		itBorder++;
	}
	itNextBorder = itBorder;
	itNextBorder++;
	while (itBorder->index < end)
	{
		if (itNextBorder->index - itBorder->index > minLength)
		{
			return itBorder;
		}
		itBorder++;
		itNextBorder++;
	}
	return borderChanges.end();
}

int cCommercialScanner::CompareLogos(tLogoChange & logo1, tLogoChange & logo2)
{
	if (logo1.logoIndex == -1 &&
		logo2.logoIndex == -1)
	{
		return 0;
	}
	else
	{
		if (logo1.logoIndex == logo2.logoIndex)
		{
			if (abs(logo1.logoPosition.x - logo2.logoPosition.x) > 10 ||
				abs(logo1.logoPosition.y - logo2.logoPosition.y) > 10)
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			if (logo1.logoIndex == -1 ||
				logo2.logoIndex == -1)
			{
				return 3;
			}
			else
			{
				return 2;
			}
		}
	}
}
int cCommercialScanner::CompareBorders(tBorder & border1, tBorder & border2)
{
	int onePixelChange = 0;
	int twoPixelChange = 0;
	int threePixelChange = 0;
	int morePixelChange = 0;
	int pixelChange[4];
	int i;

	pixelChange[0] = abs(border2.top - border1.top);
	pixelChange[1] = abs(border2.bottom - border1.bottom);
	pixelChange[2] = abs(border2.left - border1.left);
	pixelChange[3] = abs(border2.right - border1.right);
	for (i = 0; i < 4; i++)
	{
		if (pixelChange[i] > 0)
		{
			if (pixelChange[i] > 1)
			{
				if (pixelChange[i] > 2)
				{
					if (pixelChange[i] > 3)
					{
						morePixelChange++;
					}
					else
					{
						threePixelChange++;
					}
				}
				else
				{
					twoPixelChange++;
				}
			}
			else
			{
				onePixelChange++;
			}
		}
	}
	if (morePixelChange > 0 ||
//		threePixelChange > 1 ||
		twoPixelChange > 2)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

void cCommercialScanner::DumpChanges()
{
	list<tLogoChange>::iterator itLogo;
	list<tBorderChange>::iterator itBorder;
	list<tBlackWhiteChange>::iterator itBlackWhite;
	list<tAC3ChannelChange>::iterator itAC3;
	list<tMonoStereoChange>::iterator itMonoStereo;
	list<tChange>::iterator itChange;

	fprintf (this->logFile, "LogoChanges\n");
	for (itLogo = logoChanges.begin(); itLogo != logoChanges.end(); itLogo++)
	{
		if (itLogo->logoIndex == -1)
			fprintf (this->logFile, "%s %4d\n", IndexToHMSF(itLogo->index, true), itLogo->logoIndex);
		else
			fprintf (this->logFile, "%s %4d %4d %4d\n", IndexToHMSF(itLogo->index, true), itLogo->logoIndex,
														itLogo->logoPosition.x, itLogo->logoPosition.y);
	}
	fprintf (this->logFile, "BorderChanges\n");
	for (itBorder = borderChanges.begin(); itBorder != borderChanges.end(); itBorder++)
	{
		fprintf (this->logFile, "%s %4d%4d%4d%4d\n", IndexToHMSF(itBorder->index, true),
													 itBorder->border.top, itBorder->border.bottom,
													 itBorder->border.left, itBorder->border.right);
	}
	fprintf (this->logFile, "BWChanges\n");
	for (itBlackWhite = bwChanges.begin(); itBlackWhite != bwChanges.end(); itBlackWhite++)
	{
		fprintf (this->logFile, "%s %4d\n", IndexToHMSF(itBlackWhite->index, true), itBlackWhite->bwState);
	}
	fprintf (this->logFile, "AC3Changes\n");
	for (itAC3 = ac3Changes.begin(); itAC3 != ac3Changes.end(); itAC3++)
	{
		fprintf (this->logFile, "%s %4d\n", IndexToHMSF(itAC3->index, true), itAC3->channelMode);
	}
	fprintf(this->logFile, "MonoStereoChanges\n");
	for (itMonoStereo = monostereoChanges.begin(); itMonoStereo != monostereoChanges.end(); itMonoStereo++)
	{
		fprintf(this->logFile, "%s %4d\n", IndexToHMSF(itMonoStereo->index, true), itMonoStereo->channels);
	}
	fprintf(this->logFile, "Changes\n");
	for (itChange = changes.begin(); itChange != changes.end(); itChange++)
	{
		fprintf(this->logFile, "%s  %d\n", IndexToHMSF(itChange->index, true), itChange->state);
	}

	if (kVerbose)
	{
		fprintf(stdout, "LogoChanges\n");
		for (itLogo = logoChanges.begin(); itLogo != logoChanges.end(); itLogo++)
		{
			if (itLogo->logoIndex == -1)
				fprintf(stdout, "%s %4d\n", IndexToHMSF(itLogo->index, true), itLogo->logoIndex);
			else
				fprintf(stdout, "%s %4d %4d %4d\n", IndexToHMSF(itLogo->index, true), itLogo->logoIndex,
													itLogo->logoPosition.x, itLogo->logoPosition.y);
		}
		fprintf(stdout, "BorderChanges\n");
		for (itBorder = borderChanges.begin(); itBorder != borderChanges.end(); itBorder++)
		{
			fprintf(stdout, "%s %4d%4d%4d%4d\n", IndexToHMSF(itBorder->index, true),
												 itBorder->border.top, itBorder->border.bottom,
												 itBorder->border.left, itBorder->border.right);
		}
		fprintf(stdout, "BWChanges\n");
		for (itBlackWhite = bwChanges.begin(); itBlackWhite != bwChanges.end(); itBlackWhite++)
		{
			fprintf(stdout, "%s %4d\n", IndexToHMSF(itBlackWhite->index, true), itBlackWhite->bwState);
		}
		fprintf(stdout, "AC3Changes\n");
		for (itAC3 = ac3Changes.begin(); itAC3 != ac3Changes.end(); itAC3++)
		{
			fprintf(stdout, "%s %4d\n", IndexToHMSF(itAC3->index, true), itAC3->channelMode);
		}
		fprintf(stdout, "MonoStereoChanges\n");
		for (itMonoStereo = monostereoChanges.begin(); itMonoStereo != monostereoChanges.end(); itMonoStereo++)
		{
			fprintf(stdout, "%s %4d\n", IndexToHMSF(itMonoStereo->index, true), itMonoStereo->channels);
		}
		fprintf(stdout, "Changes\n");
		for (itChange = changes.begin(); itChange != changes.end(); itChange++)
		{
			fprintf(stdout, "%s  %d\n", IndexToHMSF(itChange->index, true), itChange->state);
		}
	}
}

void cCommercialScanner::SaveChanges(int prefix)
{
	char filename[100];

	sprintf(filename, "%s/%02dclogo.dat", kFilePath, prefix);
	FILE * logoFile = fopen(filename, "wb");
	list<tLogoChange>::iterator itLogo;
	for (itLogo = logoChanges.begin(); itLogo != logoChanges.end(); itLogo++)
	{
		fwrite(itLogo.operator->(), sizeof(tLogoChange), 1, logoFile);
	}
	fclose(logoFile);

	sprintf(filename, "%s/%02dcborder.dat", kFilePath, prefix);
	FILE * borderFile = fopen(filename, "wb");
	list<tBorderChange>::iterator itBorder;
	for (itBorder = borderChanges.begin(); itBorder != borderChanges.end(); itBorder++)
	{
		fwrite(itBorder.operator->(), sizeof(tBorderChange), 1, borderFile);
	}
	fclose(borderFile);

	sprintf(filename, "%s/%02dcblack.dat", kFilePath, prefix);
	FILE * bwFile = fopen(filename, "wb");
	list<tBlackWhiteChange>::iterator itBlackWhite;
	for (itBlackWhite = bwChanges.begin(); itBlackWhite != bwChanges.end(); itBlackWhite++)
	{
		fwrite(itBlackWhite.operator->(), sizeof(tBlackWhiteChange), 1, bwFile);
	}
	fclose(bwFile);

	sprintf(filename, "%s/%02dcac3.dat", kFilePath, prefix);
	FILE * ac3File = fopen(filename, "wb");
	list<tAC3ChannelChange>::iterator itAC3;
	for (itAC3 = ac3Changes.begin(); itAC3 != ac3Changes.end(); itAC3++)
	{
		fwrite(itAC3.operator->(), sizeof(tAC3ChannelChange), 1, ac3File);
	}
	fclose(ac3File);
}

void cCommercialScanner::RefreshFeature(int index, cFeature *feature, void * frame)
{
	feature->Refresh(frame);
	if (feature->Changed())
	{
		fprintf(stdout, "%s    %s\n", IndexToHMSF(index, true), feature->Dump());
		fprintf(this->logFile, "%s    %s\n", IndexToHMSF(index, true), feature->Dump());
	}
}

void cCommercialScanner::CheckFeature(int index, cFeature *feature)
{
	if (feature->Changed())
	{
		fprintf(stdout, "%s    %s\n", IndexToHMSF(index, true), feature->Dump());
		fprintf(this->logFile, "%s    %s\n", IndexToHMSF(index, true), feature->Dump());
	}
}
