/*
 * stream.cpp: Data stream with bitwise access
 *
 * See the main source file 'main.cpp' for copyright information and
 * how to reach the author.
 */

#include "stream.h"
#include "util.h"

#include <stdlib.h>

cBitStream::cBitStream(unsigned char *data, unsigned int length)
:	data(data),
	length(length),
	isOnEnd(false),
	byteOffset(0),
	bitOffset(0)
{
	if (this->length == 0)
		isOnEnd = true;
}

cBitStream::~cBitStream()
{
}

void cBitStream::SeekStart()
{
	byteOffset = 0;
	bitOffset = 0;
	if (this->length == 0)
		isOnEnd = true;
	else
		isOnEnd = false;
}

void cBitStream::SeekBytes(unsigned int bytes)
{
	byteOffset += bytes;
	if (byteOffset >= length)
	{
		byteOffset = length - 1;
		isOnEnd = true;
	}
}

void cBitStream::SeekBits(unsigned int bits)
{
	byteOffset += bits / 8;
	bitOffset += bits % 8;
	if (bitOffset >= 8)
	{
		byteOffset += 1;
		bitOffset -= 8;
	}
	if (byteOffset >= length)
	{
		byteOffset = length - 1;
		isOnEnd = true;
	}
}

void cBitStream::AlignBits()
{
	if (bitOffset > 0)
	{
		bitOffset = 0;
		byteOffset++;
	}
	if (byteOffset >= length)
	{
		byteOffset = length - 1;
		isOnEnd = true;
	}
}

unsigned int cBitStream::NextBits(unsigned int bits)
{
	unsigned int byte = byteOffset;
	unsigned int bit = bitOffset;
	unsigned int value = GetBits(bits);
	byteOffset = byte;
	bitOffset = bit;
	return value;
}

unsigned int cBitStream::GetBits(unsigned int bits)
{
	if (bits <= 32)
	{
		unsigned int value;
		unsigned int index;

		value = GetBit();
		for (index = 1; index < bits; index++)
		{
			value <<= 1;
			value |= GetBit();
		}
		if (byteOffset >= length)
		{
			byteOffset = length - 1;
			isOnEnd = true;
		}
		return value;
	}
	return 0;
}

unsigned int inline cBitStream::GetBit()
{
	if (byteOffset < length)
	{
		unsigned int bit = (data[byteOffset] >> (7 - bitOffset)) & 1;
		bitOffset++;
		if (bitOffset == 8)
		{
			byteOffset++;
			bitOffset = 0;
		}
		return bit;
	}
	return 0;
}
