/*
 * stream.h: Data stream with bitwise access
 *
 * See the main source file 'main.cpp' for copyright information and
 * how to reach the author.
 */

#ifndef _STREAM_H_
#define _STREAM_H_

/**
 * A class representing a data stream with bitwise access.
 */
class cBitStream
{
public:
	/**
	 * Constructs the data stream. The caller is responsible for allocating and deallocating the buffer, data is pointing at.
	 *
	 * @param data pointer to the buffer
	 * @param length length of the buffer
	 */
	cBitStream(unsigned char *data, unsigned int length);
	/**
	 * Destructs the data stream. Actually does nothing.
	 */
	~cBitStream();
	/**
	 * Retrieves the length of the stream.
	 *
	 * @return length in bytes
	 */
	unsigned int Length() { return length; }
	/**
	 * Retrieves whether internal stream pointer is at the end of the stream or not.
	 *
	 * @return true if internal stream pointer is at the end of the stream, false if not.
	 */
	bool IsOnEnd() { return isOnEnd; }
	/**
	 * Moves internal stream pointer to the start of the stream.
	 */
	void SeekStart();
	/**
	 * Moves internal stream pointer bytewise forward.
	 * 
	 * @param bytes bytes to move in stream
	 */
	void SeekBytes(unsigned int bytes);
	/**
	 * Moves internal stream pointer bitwise forward.
	 * 
	 * @param bits bits to move in stream
	 */
	void SeekBits(unsigned int bits);
	/**
	 * Moves internal stream pointer to next byte-aligned position.
	 */
	void AlignBits();
	/**
	 * Retrieves next bits of data. Internal stream pointer is not moved.
	 *
	 * @param bits length of data in bits, a maximum of 32 is allowed.
	 *
	 * @return a dword containing the next bits of data
	 */
	unsigned int NextBits(unsigned int bits);
	/**
	 * Retrieves next bits of data. Internal stream pointer is moved.
	 *
	 * @param bits length of data in bits, a maximum of 32 is allowed.
	 *
	 * @return a dword containing the next bits of data
	 */
	unsigned int GetBits(unsigned int bits);
	/**
	 * Retrieves next single bit of data. Internal stream pointer is moved.
	 *
	 * @return a dword containing the next bit of data
	 */
	unsigned int inline GetBit();
protected:
	unsigned char *data;
	unsigned int length;
	bool isOnEnd;
	unsigned int byteOffset;
	unsigned int bitOffset;
};

#endif

