/*
 * util.cpp: Various utility functions
 *
 * See the main source file 'main.cpp' for copyright information and
 * how to reach the author.
 */

#include "util.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>

unsigned short bswaps (unsigned short s)
{
	s = (s >> 8) | (s << 8);
	return s;
}

unsigned long bswapl (unsigned long l)
{
	l = (l >> 24) |
		((l >> 8) & 0xFF00) |
		((l << 8) & 0xFF0000) |
		(l << 24);
	return l;
}

static const char kVDRHost[] = "127.0.0.1";
static const short kVDRPort = 2001;

int VDRMessage(const char * message)
{
	int s;
	struct sockaddr_in addr;
	int err;
	char str[100];

	s = socket(PF_INET, SOCK_STREAM, 0);
	if (s == -1)
	{
		err = errno;
		return err;
	}

	addr.sin_family = AF_INET;
	addr.sin_port = htons(kVDRPort);
	addr.sin_addr.s_addr = inet_addr(kVDRHost);

	err = connect(s, (struct sockaddr *) &addr, sizeof (struct sockaddr));
	if (err == -1)
	{
		err = errno;
		close(s);
		return err;
	}

	err = recv(s, str, 100, 0);
	if (err == -1)
	{
		err = errno;
		close(s);
		return err;
	}

	int len = sprintf(str, "mesg %s\n", message);
	err = (int) send(s, str, len, 0);
	if (err == -1)
	{
		err = errno;
		close(s);
		return err;
	}

	err = close(s);
	if (err == -1)
	{
		err = errno;
		return err;
	}

	return 0;
}

